# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Frabjous. If not, see <http://www.gnu.org/licenses/>.
"""RSS and/or Atom feeds for frabjous.blog."""

from django.contrib.syndication.feeds import Feed
from django.utils.feedgenerator import Atom1Feed, Rss201rev2Feed
from django.core.exceptions import ObjectDoesNotExist
from frabjous.blog.models import Post
from frabjous.categories.models import Category

class RSSLatestEntries(Feed):
    """Latest `Posts` as an RSS feed."""

    title = "Latest Blog Entries"
    link = "/blog/"
    description = "Latest entries from the blog"
    copyright = "Copyright (c) some author 2009"
    title_template = 'blog/feeds/rss_title.html'
    description_template = 'blog/feeds/rss_description.html'
    feed_type = Rss201rev2Feed

    def items(self):
        """See `Feed`."""
        return Post.objects.get_displayable_posts().order_by(
            '-date_posted')[:5]

    def item_cateories(self, item):
        """See `Feed`."""
        return item.categories.all()

    def item_pubdate(self, item):
        """See `Feed`."""
        return item.date_posted

    def item_author_name(self, item):
        """See `Feed`."""
        # We try full_name then username
        if item.author.get_full_name():
            return item.author.get_full_name()
        else:
            return item.author.username

class AtomLatestEntries(RSSLatestEntries):
    """Adapt RSSLatestEntries to provide an Atom feed."""

    feed_type = Atom1Feed

class CategoryPostFeed(RSSLatestEntries):
    """An RSS feed of all the `Post`s in a `Category`."""

    def get_object(self, bits):
        """See `Feed`."""
        if len(bits) != 1:
            raise ObjectDoesNotExist

        return Category.objects.get(slug__exact=bits[0])

    def title(self, obj):
        """See `Feed`."""
        return "Blog Posts in %s" % obj.name.title()

    def link(self, obj):
        """See `Feed`."""
        return "/blog/categories/%s/" % obj.slug

    def description(self, obj):
        """See `Feed`."""
        return "Latest blog entries in category '%s'" % obj.name.title()

    def items(self, obj):
        """See `Feed`."""
        return Post.objects.get_by_category(category_obj=obj)


class ExcludeCategoryPostFeed(CategoryPostFeed):
    """An RSS feed for all `Post`s not in a category."""

    def title(self, obj):
        """See `Feed`."""
        return "Blog Posts not in %s" % obj.name.title()

    def link(self, obj):
        """See `Feed`."""
        return "/blog/"

    def description(self, obj):
        """See `Feed`."""
        return "Latest blog entries not in category '%s'" % obj.name.title()

    def items(self, obj):
        """See `Feed`."""
        latest_posts = Post.objects.get_most_recent()
        non_cat_posts = [
            post for post in latest_posts if obj not in post.categories.all()]

        return non_cat_posts
