# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.
"""Metaweblog API implementation for frabjous.blog."""

from django.contrib.admin.models import ADDITION, LogEntry
from django.contrib.auth import authenticate
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site

from django_xmlrpc.decorators import xmlrpc_func

from frabjous.blog.models import Post
from frabjous.categories.models import Category


class MetaWeblogError(Exception):
    """Raised whenever something goes wrong with a MetaWeblog doodad."""


def mw_authenticate(username, password):
    """Authenticate a metaweblog user.

    :param username: The user to auth.
    :param password: Their password.

    If authentication is successful a `User` instance will be returned.
    Otherwise a MetaWeblogError will be raised.
    """
    user = authenticate(username=username, password=password)
    if not user or not user.has_perm('change_post'):
        raise MetaWeblogError(
            "Sorry, you're not authorised to access that post")
    else:
        return user


@xmlrpc_func(
    name='metaWeblog.getPost', returns='struct',
    args=['int', 'string', 'string'])
def get_post(postid, username, password):
    """Retrieve a Post by ID and return it.

    :param postid: The ID of the Post to retrieve.
    :param username: The Django username being used for authentication
        and authorisation.
    :param password: The Django password to go with username.

    """
    # First, authenticate them
    user = mw_authenticate(username, password)

    # Try to load the post
    try:
        post = Post.objects.get(pk=int(postid))
    except DoesNotExist, e:
        raise MetaWeblogError("The post you were looking for doesn't exist")

    # Turn the post into a metaweblog struct
    # TODO: Implement this - RSS syndication mayhap?
    return post.as_metaweblog_struct()


@xmlrpc_func(
    name='metaWeblog.getRecentPosts', returns='list',
    args=['int', 'string', 'string', 'int'])
def get_recent_posts(blogid, username, password, number_of_posts=5):
    """Return the latest posts for a given blog.

    :param blogid: The ID of the blog to retrieve from. Currently
        ignored as we don't have multiple blogs (yet).
    :param username: The username to authenticate against.
    :param password: The password for username.
    :param number_of_posts: The number of posts to return.
    """
    # Authenticate. This will raise an error on failure that we don't
    # handle.
    user = mw_authenticate(username, password)

    posts = Post.objects.order_by('-date_posted')[:number_of_posts]
    return [post.as_metaweblog_struct() for post in posts]


@xmlrpc_func(
    name='metaWeblog.getUsersBlogs', returns='list',
    args=['string', 'string', 'string'])
def get_users_blogs(appkey, username, password):
    """Return all blogs accessible to a user.

    :param appkey: Not used.
    :param username: The username to authenticate against.
    :param password: The password for username.
    """
    user = mw_authenticate(username, password)

    # This is a hack
    # TODO: Fix it!
    return [{
        'isAdmin': user.is_superuser,
        'url': 'http://%s' % Site.objects.get_current().domain,
        'blogid': 1,
        'blogName': 'Frabjous blog'}]


@xmlrpc_func(
    name='metaWeblog.newPost', returns='int',
    args=['int', 'string', 'string', 'struct', 'boolean'])
def new_post(blogid, username, password, struct, publish):
    """Add a new Post to a Blog.

    :param blogid: The ID of the blog to which the post will be added
        (ignored at present).
    :param username: The username to use for authentication.
    :param password: The password to go with username.
    :param struct: A metaweblog API struct containing the new Post data.
    :param publish: A value indicating whether or not the post should be
        published.
    """
    user = mw_authenticate(username, password)

    # We ignore blogid, but if we didn't we'd use it here.
    if not user.has_perm('add_post'):
        raise MetaWeblogError(
            "Sorry, you don't have permission to add a Post")

    # We let Post do the work
    try:
        post = Post.objects.create_from_metaweblog_struct(struct, user)
    except Exception, e:
        raise MetaWeblogError("Unable to create new post: %s" % str(e))

    # Mark the post as published if that's what's required
    if publish:
        post.published = True
        post.save()

    # Add a log entry for this action.
    content_type = ContentType.objects.get(name='post')
    LogEntry.objects.log_action(
        user_id=user.id, content_type_id=content_type.id, object_id=post.id,
        object_repr=post.subject, action_flag=ADDITION)

    return post.id


@xmlrpc_func(
    name='metaWeblog.getCategories', returns='list',
    args=['int', 'string', 'string'])
def get_categories(blogid, username, password):
    """Return the categories for a given blog.

    :param blogid: Ignored.
    :param username: The username for authentication.
    :param password: The password for same.
    """
    user = mw_authenticate(username, password)

    # For now we just return a list of the categories available.
    cat_list = []
    domain = Site.objects.get_current().domain
    for cat in Category.objects.all():
        cat_dict = {
            'description': cat.name,
            'htmlUrl': 'http://%s/blog%s' % (
                domain, cat.get_absolute_url()),
            'rssUrl': 'http://%s/blog/feeds/%s' % (
                domain, cat.get_absolute_url())}
        cat_list.append()
    return cat_list


@xmlrpc_func(
    name='metaWeblog.editPost', returns='struct',
    args=['int', 'string', 'string', 'struct', 'boolean'])
def edit_post(postid, username, password, struct, publish):
    """Update an extant post with new data from a metaWeblog struct.

    :param postid: The id of the post to update.
    :param username: The username to use to update the post.
    :param password: The password for `username`.
    :param publish: Whether or not the post should be marked as
        published.
    """
    user = mw_authenticate(username, password)

    # If we can load the Post, we can update it with the new data
    try:
        post = Post.objects.get(pk=int(postid))
        post.populate_from_metaweblog_struct(struct, user)
        return True
    except Exception, e:
        raise MetaWeblogError("Unable to edit post: %s", str(e))
