# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.
"""Database models for the Frabjous blog application."""

from datetime import datetime, timedelta

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models import Q
from django.db.models.signals import post_save
from django.dispatch import dispatcher
from django.template.defaultfilters import slugify

from frabjous.categories.models import Category
from frabjous.comments.models import Comment, ContentType
from frabjous.licensing.models import License
from frabjous.media.models import MediaObject
from frabjous.rpc.models import Pingback, RPCPingServer
from frabjous.rpc.util import rpc_ping


# Set up the DEFAULT_LICENSE constant. If there's one defined, use it.
# If not, use None. Yes, this is a hack.
try:
    DEFAULT_LICENSE = settings.DEFAULT_LICENSE
except AttributeError:
    DEFAULT_LICENSE = None

class PostManager(models.Manager):
    """A custom manager for Posts."""

    def create_from_metaweblog_struct(self, struct, user):
        """Create a Post object from a metaweblog struct and return it.

        :param struct: The metaweblog struct to use to create the Post.
        :param user: The User that is creating the post.
        """
        post = self.model()
        post.populate_from_metaweblog_struct(struct, user)
        return post

    def get_by_category(self, *args, **kwargs):
        """Return all Posts for a given category.

        :param category_slug: The category slug to look up. This can
            also be passed as the first positional argument (for
            backwards compatibility
        :param category_obj: The Category object to search on.
        :param with_children: If true, all child categories of the
            category will also be searched
        """
        if kwargs.get('category_obj', False):
            cat = kwargs['category_obj']
        elif kwargs.get('category_slug', False):
            cat = Category.objects.get(slug=kwargs['category_slug'])
        elif len(args) == 1:
            cat = Category.objects.get(slug=args[0])
        else:
            raise ObjectDoesNotExist

        q = Q(categories__exact=cat)

        if kwargs.get('with_children', False):
            # Loop through the category's children and add them to our Q
            for child_category in cat.children.all():
                q = q | Q(categories__exact=child_category)

        # We can get our list of posts using the q now...
        return self.filter(q).distinct()

    def get_displayable_posts(self, sites=None):
        """Return the displayable posts."""
        if sites is None:
            sites = Site.objects.get_current()

        results = self.filter(
            published=True, sites=sites)
        return results

    def get_most_recent(self, number=5):
        """Return. `number` most recent posts in reverse-date order.

        :param number: The number of posts to return.
        """
        recent_posts = self.get_displayable_posts()[:number]

        return recent_posts


class Post(models.Model):
    """Represents a single post in the blog."""

    author = models.ForeignKey(User)
    body = models.TextField()
    categories = models.ManyToManyField(Category, blank=True)
    comments_enabled = models.BooleanField(default=True)
    excerpt = models.TextField(null=True, blank=True)
    date_modified = models.DateTimeField(
        auto_now=True, null=True, blank=True)
    date_posted = models.DateTimeField(default=datetime.now)
    date_published = models.DateTimeField(null=True, blank=True)
    default_enclosure = models.ForeignKey(
        MediaObject, null=True, blank=True, related_name='enclosing_post')
    license = models.ForeignKey(License, default=DEFAULT_LICENSE)
    media_objects = models.ManyToManyField(MediaObject, blank=True)
    objects = PostManager()
    published = models.BooleanField(default=False)
    rpc_servers_pinged = models.ManyToManyField(RPCPingServer, blank=True)
    slug = models.SlugField(
        unique_for_date='date_posted', max_length=100)
    subject = models.CharField(max_length=255, null=True, blank=True)
    sites = models.ManyToManyField(
        Site, related_name='posts', default=str(settings.SITE_ID))

    @property
    def comments(self):
        """Return the list of comments for this post."""
        ct = ContentType.objects.get(name='post')
        comments = Comment.objects.filter(
            content_type=ct, object_id=self.id, is_authorised=True,
            is_visible=True, is_spam=False).order_by('date')

        return comments

    @property
    def pingbacks(self):
        """Return the list of pingbacks for this post."""
        post_content_type = ContentType.objects.get(name='post')
        pingbacks = Pingback.objects.filter(
            content_type=post_content_type, object_id=self.id)

        return pingbacks.order_by('date_added')

    @property
    def is_categorised(self):
        """Return True if a post is categorised."""
        if self.categories is None or len(self.categories.all()) < 1:
            return False
        else:
            return True

    def add_rpc_pinged(self, srv):
        """Add an RPCPingServer to the list of pinged servers.

        :param srv: The RPCPingServer instance to add.
        """
        self.rpc_servers_pinged.add(srv)
        self.save()

    def rpc_server_pinged(self, srv):
        """Check whether or not an RPC Server has been pinged by a post."""
        return srv in self.rpc_servers_pinged.all()

    def get_extra_categories(self):
        """Return the 'extra' categories for a Post.

        Extra categories are calculated by resolving the hierarchy of
        categories for each category assigned to a post.
        """
        ext_cats = []
        for cat in self.categories.all():
            for ancestor in cat.get_lineage():
                if ancestor not in ext_cats:
                    ext_cats.append(ancestor)

        ext_cats.sort()
        return ext_cats

    def get_contacts(self):
        """Fetch the list of stakeholders for a post.

        :return: A list of Users to be contacted
        """
        # At the moment we just return the Post's author
        # TODO: Perhaps expand this
        return [self.author,]

    def get_absolute_url(self):
        """Return the absolute URL of the post."""
        # If we don't have a slug to go with, we go with the ID of the post
        if self.slug is not None:
            slug = self.slug
        else:
            slug = self.id

        return "/blog/%i/%s/%s/%s/" % (
            self.date_posted.year,
            str(self.date_posted.month).zfill(2),
            str(self.date_posted.day).zfill(2),
            slug,
            )

    def get_permalink(self):
        """Return the permalink for the Post, including the Site domain."""
        return 'http://%s%s' % (
            Site.objects.get_current().domain, self.get_absolute_url())

    def __unicode__(self):
        return self.subject

    def as_metaweblog_struct(self):
        """Convert a POST to a MetaWeblog API struct."""
        categories = [str(cat) for cat in self.categories.all()]

        struct = {
            'title': self.subject,
            'link': "http://%s%s" % (
                Site.objects.get_current().domain, self.get_absolute_url()),
            'description': self.body,
            'author': self.author.email,
            'comments': "http://%s%s" % (
                Site.objects.get_current().domain, self.get_absolute_url()),
            'guid': "http://%s%s" % (
                Site.objects.get_current().domain, self.get_absolute_url()),
            'pubDate': self.date_posted.strftime("%Y-%m-%d %H:%M:%S"),
            'category': categories,
            'postid': self.id,
        }

        return struct

    def populate_from_metaweblog_struct(self, struct, user):
        """Populate a post from a metaweblog struct.

        :param struct: The struct to use for population.
        :param user: The user doing the work.
        """
        self.subject = struct.get('title')
        self.body = struct.get('description')
        self.date_posted = struct.get('pubDate')
        self.author = user
        self.date_posted = datetime.now()

        # We only alter the slug if it's not None
        if not self.slug:
            self.slug = slugify(self.subject)

        # We clear out the categories and repopulate the list
        try:
            if self.categories.count() > 0:
                self.categories.clear()
        except ValueError:
            # We ignore ValueErrors because if they occur it's because this is
            # a new Post - therefore they aren't worth the worry.
            pass
        self.save()

        # Add categories to the Post if we can
        for category in struct.get('categories', []):
            # We do this rather than using get_or_create because we only want to
            # search on a name, not a slug.
            try:
                cat = Category.objects.get(name__iexact=category)
            except:
                cat = Category(name=category, slug=slugify(category))
                cat.save()
            self.categories.add(cat)

        self.save()

    class Meta:
        ordering = ('-date_posted',)


class PostByEmailAliasManager(models.Manager):
    """A manager for email aliases."""

    def find_user_by_email(self, email):
        """Find and return a user based on their email address or an alias."""
        # First, try and find the user by their alias:
        try:
            alias = self.get(email=email)
            return alias.user
        except PostByEmailAlias.DoesNotExist:
            pass

        # If we can't find them by alias, we try to find them by email
        # address. If that doesn't work we let User.DoesNotExist travel
        # up-stack.
        return User.objects.get(email=email)


class PostByEmailAlias(models.Model):
    """An email alias from which a User can post."""

    user = models.ForeignKey(User, related_name='email_posting_aliases')
    email = models.EmailField()

    objects = PostByEmailAliasManager()

    class Admin:
        list_display = ('user', 'email')


def clear_post_cache(**kwargs):
    """Clear the post cache so that new posts always appear."""
    cache.delete('blog.Post.recent_posts')


# Connect up the signals
post_save.connect(clear_post_cache, sender=Post)
post_save.connect(rpc_ping, sender=Post)
