# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.

"""Views for the blog app."""

from django.core.exceptions import ObjectDoesNotExist
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.http import Http404, HttpResponseNotFound, HttpResponseRedirect
from django.shortcuts import render_to_response as render
from django.template import Context, RequestContext, Template
from django.template.loader import get_template
from django.views.generic.list_detail import object_list

from frabjous.blog.models import Post
from frabjous.categories.models import Category
from frabjous.comments.forms import build_comment_form, CommentForm
from frabjous.comments.views import (
    object_detail_with_comment_form, AddCommentView,
    InvalidCommentFormError)
from frabjous.comments.models import Comment
from frabjous.rpc.util import add_pingback_header


def category_posts(request, category, *args, **kwargs):
    """Shows a list of posts for a given category and its children."""
    # ... and then pass the buck to object_list, since it's better at
    # rendering object lists than we are...
    context = {'extURL': 'categories/%s/' % category,}

    cat_posts = Post.objects.get_by_category(category).filter(published=True)
    extra_context = dict(
        context, category=Category.objects.get(slug=category))
    object_list_dict = dict(
        kwargs, queryset=cat_posts, paginate_by=5,
        template_name='blog/catpostlist.html', extra_context=extra_context)

    return object_list(request, *args, **object_list_dict)


def single_post(request, *args, **kwargs):
    """Display a single post based on its Year, Month, Day and slug.

    Uses `frabjous.comments.views.object_detail_with_comment_form() to render
    the data and adds the appropriate X-Pingback header.
    """
    response = add_pingback_header(
        object_detail_with_comment_form(request, *args, **kwargs))

    return response


def single_post_by_id(request, post_id):
    """Display a single post by its ID."""
    try:
        post = Post.objects.get(pk=post_id)
    except ObjectDoesNotExist, error:
        raise Http404("The post you're looking for doesn't exist.")

    # If the visitor is a logged in user then they can see unpublished
    # posts. Otherwise, they can't.
    if not request.user.is_authenticated() and not post.published:
        response = render(
            '403.html',
            {'message': "You don't have permission to access that post."},
            context_instance=RequestContext(request))
        response.status = 403
        return response

    response = render(
        'blog/post.html',
        {'object': post, 'comment_form': build_comment_form(request)},
        context_instance=RequestContext(request))

    # Add the pingback header.
    response = add_pingback_header(response)
    return response


def add_comment(request, year, month, day, slug):
    """Add a comment to a Post."""
    # First, try to load the post. There's little point going any
    # further if we can't.
    try:
        post = Post.objects.get(
            slug=slug, date_posted__year=int(year),
            date_posted__month=int(month), date_posted__day=int(day))
    except Post.DoesNotExist:
        raise Http404()

    if request.method == 'GET':
        return HttpResponseRedirect(post.get_absolute_url())

    # We pass the post to frabjous.comments.views.AddCommentView, which will
    # do all the work.
    add_comment_view = AddCommentView(request, 'blog/post.html', post)
    return add_comment_view.execute()
