# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.
"""Database models for the categories app"""

from django.conf import settings
from django.contrib import admin
from django.core.cache import cache
from django.db import models
from django.db.models import signals
from django.dispatch import dispatcher


class CategoryAdmin(admin.ModelAdmin):

    prepopulated_fields = {'slug': ("name",)}


class Category(models.Model):
    """Represents a single category for an assent (blog post, etc.)"""

    name = models.CharField(max_length=255,unique=True)
    parent = models.ForeignKey(
        'self', null=True, blank=True, related_name='children')
    ref_count = models.IntegerField(default=0)
    show_in_main_feed = models.BooleanField(default=True)
    slug = models.SlugField(unique=True)

    @property
    def popularity(self):
        """Return the popularity of the category as a string."""
        popularity_thresholds = settings.get(
            'CATEGORY_POPULARITY_THRESHOLDS', {})

        # Reverse the threshold list so that we do highest-first.
        thresholds = popularity_thresholds.keys()
        thresholds.sort()
        thresholds.reverse()

        # Check the ref_count against the thresholds and return the
        # correct class.
        for i in thresholds:
            if self.ref_count >= i:
                return popularity_thresholds[i]

    def update_ref_count(self):
        """Update the reference count of the current category."""
        ref_count = 0
        # Loop through all many_to_many related fields and base our
        # count on their counts.
        for obj in self._meta.get_all_related_many_to_many_objects():
            # We can get an accessorname to work with now We put this in
            # a try..except:pass block because we don't care overmuch if
            # it fails.
            try:
                ref_count += self.__getattribute__(
                    obj.get_accessor_name()).count()
            except:
                pass

        self.ref_count = ref_count
        self.save()

    def get_lineage(self):
        """Return the lineage of a Category.

        Lineage in this context means the category's parent's parent's
        parent and so on.
        """
        parent = self.parent
        lineage = []
        while parent:
            if parent not in lineage:
                lineage.append(parent)

            parent = parent.parent

        return lineage

    def get_absolute_url(self):
        """Return the absolute URL for the category."""
        return "/categories/%s/" % self.slug

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = "Category"
        verbose_name_plural = "Categories"
        ordering = ('name',)


admin.site.register(Category, CategoryAdmin)
