# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License
# along with Frabjous. If not, see <http://www.gnu.org/licenses/>.
"""Models for the comments app."""

from datetime import datetime

from django.conf import settings
from django.contrib.admin.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import send_mail
from django.db import models
from django.db.models.signals import post_save

from frabjous.comments.util import akismet_comment_check


class NotAuthenticatedError(Exception):
    """Raised when an unauthenticated user tries to comment."""


class Comment(models.Model):
    """Represents a comment on an object."""

    author_email = models.EmailField(
        null=True, blank=True, verbose_name="Your email address")
    author_ip = models.IPAddressField()
    author_name = models.CharField(
        verbose_name="Your name", max_length=255, null=True, blank=True)
    author_openid = models.URLField(null=True, blank=True)
    author_url = models.URLField(null=True, blank=True)
    body = models.TextField(verbose_name="Your comment")
    content_type = models.ForeignKey(ContentType, null=True, blank=True)
    date = models.DateTimeField(default=datetime.now)
    in_reply_to = models.ForeignKey('self', null=True, blank=True)
    is_authorised = models.BooleanField(default=False)
    is_spam = models.BooleanField(default=False)
    is_visible = models.BooleanField(default=False)
    object_id = models.IntegerField(null=True, blank=True)
    title = models.CharField(
        verbose_name="Subject", max_length=255, blank=True, null=True)
    user = models.ForeignKey(User, null=True, blank=True)

    @property
    def author(self):
        """Return the author of the comment as a string."""
        if self.user:
            if self.user.get_full_name():
                return self.user.get_full_name()
            else:
                return self.user.username
        elif self.author_name:
            return self.author_name
        elif self.author_openid:
            return self.author_openid

    @property
    def content_object(self):
        """Return the content object to which this comment belongs.

        If the content object doesn't exist, return None.
        """
        try:
            return self.content_type.get_object_for_this_type(
                pk=self.object_id)
        except ObjectDoesNotExist:
            return None

    def __str__(self):
        return str(self.date) + ' ' + self.body[:10]

    def get_absolute_url(self):
        obj = self.content_type.model_class().objects.get(pk=self.object_id)
        return "%s#c%s" % (obj.get_absolute_url(), self.id)

    def mark_as_ham(self):
        """Mark a comment as ham, both internally and with Akismet."""
        # TODO implement this

    def get_contacts(self):
        """Return the contacts for this comment."""
        # If the author is a User, we just use their email address
        if self.user:
            return [self.user.email,]
        elif self.author_email:
            return [self.author_email,]

    def save(self, *args, **kwargs):
        """Save the Comment."""
        # Check that the comment is on an existing piece of content
        try:
            content_object = self.content_object
        except ObjectDoesNotExist, e:
            raise

        # We only do spam checks etc if the user is not a system user.
        if self.user is not None:
            self.author_name = self.user.get_full_name()
            self.author_email = self.user.email
            self.is_visible = True
            self.is_authorised = True
            self.is_spam = False
            return super(Comment, self).save()

        # You can only comment if you're logged in with OpenID (and
        # therefore author_openid is set, or you have given a name and
        # email address
        if self.author_openid is None or self.author_openid == '':
            if self.author_name is None or self.author_email is None:
                raise NotAuthenticatedError(
                    "User has not entered enough authentication data")
        else:
            # OpenID comments are visible by default.
            self.is_visible = True
            self.is_authorised = True
            return super(Comment, self).save()

        # We run the comment through akismet at this point. It's
        # unfortunate, but there's really no way to guarantee that
        # anything's not spam.
        is_spam = akismet_comment_check(self)
        if is_spam:
            self.is_authorised = False
            self.is_spam = True
            self.is_visible = False
        else:
            self.is_authorised = True

        super(Comment, self).save(*args, **kwargs)

    class Meta:
        verbose_name = 'comment'
        verbose_name_plural = 'comments'
        ordering = ('-date',)


def email_notifiable_contacts(instance, **kwargs):
    """Ping the notifiable contacts for a comment's content object.

    :param comment: The Comment to work with
    """
    content_object = instance.content_object
    comment_data = {
        'author': instance.author_name,
        'body': instance.body,
        'subject': str(instance.content_object),
        'url': 'http://%s%s' % (
            Site.objects.get_current().domain, instance.get_absolute_url()),
        }
    message = settings.NOTIFICATION_NEW_COMMENT % comment_data

    try:
        for user in content_object.get_contacts():
            # Don't keep notifying the author of the comment for crying
            # out loud.
            if user == instance.user:
                continue

            user.email_user(
                subject="New comment posted", message=message,
                from_email=settings.COMMENT_EMAIL_ADDRESS)
    except:
        # We don't worry about exceptions, which is slack, really
        pass

    # If this was an in-reply-to comment, notify the original commenter
    try:
        if instance.in_reply_to and instance.in_reply_to.get_contacts():
            send_mail(
                'Comment reply notification', message,
                settings.COMMENT_EMAIL_ADDRESS,
                instance.in_reply_to.get_contacts(), fail_silently=True)

    except Exception, error:
        print error


post_save.connect(email_notifiable_contacts, sender=Comment)

