# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.

# Create your views here.
__metaclass__ = type
__all__ = [
    'AddCommentView',
    'InvalidCommentFormError',
    'add_comment',
    'object_detail_with_comment_form',
    'user_comments',
    ]

import logging
import urllib

from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
from django.http import HttpResponseRedirect, HttpRequest
from django.shortcuts import render_to_response as render
from django.template import Context, RequestContext, Template
from django.template.loader import get_template
from django.views.generic.date_based import object_detail
from django.views.generic.list_detail import object_list

from frabjous.comments.forms import CommentForm, build_comment_form
from frabjous.comments.models import Comment


log = logging.getLogger('frabjous.comments')


class BaseView:
    """A Base View class from which all other View classes are derived."""

    def __init__(self, request, template):
        self.request = request
        self.response = None
        self.template = template

    def execute(self):
        """Process the request and return an appropriate HttpResponse."""
        raise NotImplementedError()

    @property
    def error_response(self):
        """Return the HttpResponse that should be returned on error."""
        raise NotImplementedError()

    @property
    def success_response(self): 
        """Return the HttpResponse that should be returned on success."""
        raise NotImplementedError()


class InvalidCommentFormError(Exception):
    """Raised when a comment form is invalid."""


class AddCommentView(BaseView):
    """A base class for adding comments."""

    def __init__(self, request, template, context_object):
        super(AddCommentView, self).__init__(request, template)
        self._comment_form = None
        self.context_object = context_object
        self.comment = None

    @property
    def comment_form(self):
        """Return a CommentForm bound to POST data if it exists."""
        # If we have post data and no comment form we create a comment
        # form from that post data.
        if not self._comment_form and self.request.POST:
            self._comment_form = CommentForm(self.request.POST)

        # If there's no post data we create a comment form from the
        # request and session data, if possible.
        elif not self._comment_form:
            # If we have openid data in the request, use that.
            openids = request.session.get('openids', None)
            if openids:
                openid = openids[-1]
                initial_data = {
                    'author_name': openid.sreg.get('nickname', ''),
                    'author_email': openid.sreg.get('email', ''),
                    }
            # If the user is authenticated, use their details.
            elif request.user.is_authenticated():
                initial_data = {
                    'author_name': request.user.get_full_name(),
                    'author_email': request.user.email,
                    }
            else:
                initial_data = {}

            self._comment_form = CommentForm(initial=initial_data)

        return self._comment_form

    @property
    def context_dict(self):
        return {
            'comment_form': self.comment_form,
            'object': self.context_object,
            }

    @property
    def response_context(self):
        """Return the response context for this view."""
        return RequestContext(self.request, self.context_dict)

    @property
    def error_response(self):
        """See `BaseView`."""
        return render(self.template, self.response_context)

    @property
    def success_response(self):
        """See `BaseView`."""
        return HttpResponseRedirect(self.comment.get_absolute_url())

    def create_comment_for_context_object(self):
        """Create a comment for a given object.

        :param context_object: The object for which the comment should be created.
        :return: The created comment.
        """
        comment_form = self.comment_form
        if not comment_form.is_valid():
            raise InvalidCommentFormError(
                "Invalid comment form. Can't create comment.")

        request = self.request
        context_object = self.context_object

        comment_data = comment_form.cleaned_data
        content_type = ContentType.objects.get(
            name=context_object._meta.object_name.lower())

        # Build the comment from the form.
        user = None
        openids = request.session.get('openids', None)
        openid = None
        if request.user.is_authenticated():
            user = request.user
        elif openids:
            openid = openids[-1]

        comment_kwargs = dict(
            comment_data, content_type=content_type,
            object_id=context_object.id,
            user=user, author_openid=openid,
            author_ip=request.META['REMOTE_ADDR'])

        # We replace reply tos with actual comment objects. We fail silently
        # when these don't exist.
        # XXX 2008-05-07 gmb:
        #     Is this DTRT?
        if comment_kwargs['in_reply_to']:
            try:
                parent_comment = Comment.objects.get(
                    pk=comment_kwargs['in_reply_to'])
            except Comment.DoesNotExist:
                pass
            else:
                comment_kwargs['in_reply_to'] = parent_comment

        self.comment = Comment.objects.create(**comment_kwargs)

    def execute(self):
        """Do all the leg-work of proccessing the request."""
        try:
            self.create_comment_for_context_object()
        except InvalidCommentFormError, error:
            return self.error_response
        else:
            return self.success_response


def user_comments(request, page=1):
    """Displays the comments of a given user, whether system or OpenID auth'd"""
    context = {}
    q = Q(is_visible=True, is_authorised=True, is_spam=False)

    # Being auth'd by the system overrides all other auth methods
    openids = request.session.get('openids', None)
    if request.user.is_authenticated():
        q = q & Q(user__exact=request.user)
    elif openids:
        openid = openids[-1]
        q = q & Q(author_openid__exact=openid.openid)
    else:
        return HttpResponseRedirect('/')

    comments = Comment.objects.filter(q).order_by('-date')
    return object_list(
        request=request, queryset=comments, paginate_by=10,
        page=page, template_name='comments/usercomments.html',
        extra_context=context)


def object_detail_with_comment_form(request, *args, **kwargs):
    """Create and return a comment form.

    :param request: The HttpRequest object to use.

    The form will be populated with data in the request and / or
    session.
    """
    comment_form = build_comment_form(request)

    # Add the form to extra context if necessary (which it should be).
    try:
        kwargs['extra_context']['comment_form'] = comment_form
    except KeyError:
        kwargs['extra_context'] = {'comment_form': comment_form}

    # We pass the buck to object_detail for rendering.
    return object_detail(request, *args, **kwargs)
