# Copyright (C) 2010 Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.
"""DB Models for the Links app."""

from django.contrib import admin
from django.db.models import (
    CharField, ForeignKey, ManyToManyField, Manager, Model, SlugField,
    URLField)

from django.utils.safestring import mark_safe


class LinkSetManager(Manager):
    """A manager for LinkSets."""

    def __getitem__(self, item):
        return self.get(slug=item)


class LinkSet(Model):
    """A set of links."""
    title = CharField(max_length=255)
    slug = SlugField(unique=True)
    parent = ForeignKey(
        'self', null=True, blank=True, related_name='children')
    objects = LinkSetManager()

    @property
    def links(self):
        """Return the links in this set.

        This method provides a nice shorthand for all().
        """
        return self._links.all()

    def __unicode__(self):
        return self.title


class LinkSetAdmin(admin.ModelAdmin):
    """Admin class for the LinkSet model."""
    list_display = ('title',)
    prepopulated_fields = {'slug': ('title',)}
admin.site.register(LinkSet, LinkSetAdmin)


class Link(Model):
    """A single link to a web resource."""
    text = CharField(max_length=255)
    url = URLField(verify_exists=True)
    link_sets = ManyToManyField(LinkSet, related_name="_links")

    @property
    def as_html(self):
        """Return the link as HTML."""
        a_string = u'<a href="%(url)s" title="%(text)s">%(text)s</>'
        return mark_safe(a_string % {'url': self.url, 'text': self.text})


class LinkAdmin(admin.ModelAdmin):
    """Admin class for the Link Model."""
    list_display = ('text', 'url', 'as_html',)
    filter_horizontal = ('link_sets',)
admin.site.register(Link, LinkAdmin)
