# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.

"""Database models for the media app."""
import mimetypes

from datetime import datetime

from django.conf import settings
from django.contrib import admin
from django.contrib.auth.models import User
from django.db import models

from frabjous.categories.models import Category
from frabjous.licensing.models import License

class MediaObject(models.Model):
    """Represents a single media object within the site."""

    author = models.ForeignKey(User)
    categories = models.ManyToManyField(Category, blank=True)
    date_created = models.DateTimeField(default=datetime.now)
    file = models.FileField(upload_to='uploads/%Y/%m/%d')
    license = models.ForeignKey(License, default=settings.DEFAULT_LICENSE)
    name = models.CharField(max_length=255, null=True, blank=True)
    slug = models.SlugField(unique_for_date='date_created')

    class Meta:
        ordering = ('-date_created',)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        """Return the absolute URL to this object."""
        return "/%s" % self.file.url

    @property
    def mime_type(self):
        """Return the mimetype of the file."""
        return mimetypes.guess_type(self.file.url)[0]

    @property
    def is_image(self):
        """Return True if the file is an image, False otherwise."""
        type, subtype = self.mime_type.split('/')
        if type == 'image':
            return True
        else:
            return False


class MediaObjectAdmin(admin.ModelAdmin):
    date_hierarchy = 'date_created'
    list_display = ('name', 'date_created')
    list_display_links = ('name', 'date_created')
    list_filter = ('date_created', 'categories')
    prepopulated_fields = {'slug': ('name',)}
    filter_horizontal = ('categories',)

admin.site.register(MediaObject, MediaObjectAdmin)

