# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.
"""Database models for the RemoteFeeds app."""

import feedparser
import pickle

from datetime import datetime, timedelta
from urlparse import urlsplit

from django.conf import settings
from django.contrib.auth.models import User
from django.core.cache import cache
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import dispatcher
from django.template.defaultfilters import slugify


class RemoteFeed(models.Model):
    """Represents an RSS or atom feed that we parse and display data from"""

    interval = models.IntegerField(default=600)
    last_checked = models.DateTimeField(null=True, blank=True)
    last_updated = models.DateTimeField(null=True, blank=True)
    name = models.CharField(max_length=100)
    posts_to_show = models.IntegerField()
    url = models.URLField(unique=True, verify_exists=False)

    @property
    def next_check(self):
        last_checked = self.last_checked
        if last_checked is None:
            last_checked = datetime.now()

        return last_checked + timedelta(seconds=self.interval)

    @property
    def needs_checking(self):
        """Returns True if the feed's next check is due."""
        return self.last_checked is None or datetime.now() >= self.next_check

    def update(self):
        """Update a feed from the remote RSS / Atom XML.

        :param feed: The RemoteFeed to update.
        """
        parsed_feed = feedparser.parse(
            self.url,
            agent="frabjous FeedParser/0.2 +http://launchpad.net/frabjous")

        # Check that the feed has been updated since we last checked.
        try:
            if parsed_feed.updated <= self.last_updated:
                self.last_checked = datetime.now()
                self.save()
                return
        except:
            pass

        # We only parse the latest n entries; saves us some time.
        for post in parsed_feed.entries[:self.posts_to_show]:
            # Does the post already exist? If not, add it
            try:
                feed_post = FeedPost.objects.get(url=post.link, feed=self)
            except:
                feed_post = self.feedpost_set.create(
                    url=post.link, subject=post.title, feed=self,
                    date=datetime(*post.date_parsed[:6]))

            feed_post.save()

        self.last_checked = datetime.now()
        self.last_updated = datetime.now()
        self.save()

    def __str__(self):
        return self.name


class FeedPostManager(models.Manager):
    """A custom manager for remote feeds."""

    def get_recent(self):
        """Return the most recent posts for all remote feeds as a dict."""
        feeds = {}
        for feed in RemoteFeed.objects.all():
            posts = feed.feedpost_set.all()[:feed.posts_to_show]
            feeds[feed.name] = posts

        return feeds


class FeedPost(models.Model):
    """Holds the details of a single post from a remote feed."""

    date = models.DateTimeField()
    feed = models.ForeignKey(RemoteFeed)
    objects = FeedPostManager()
    subject = models.CharField(max_length=255, null=True)
    post_content = models.TextField(null=True, blank=True)
    url = models.URLField(unique=True)

    def __str__(self):
        return self.subject

    class Meta:
        ordering = ('-date',)


def clear_feedpost_cache(**kwargs):
    """Clear the feedpost cache after an update."""
    cache.delete('remotefeeds.RemoteFeed.recent_posts')


post_save.connect(clear_feedpost_cache, sender=FeedPost)
