# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.

"""Pingback 1.0 implementation."""

import re

from BeautifulSoup import BeautifulSoup
from datetime import datetime
from urllib2 import urlopen
from urlparse import urlsplit
from xmlrpclib import Fault

from django.conf import settings
from django_xmlrpc.decorators import xmlrpc_method

from frabjous.blog.models import Post
from frabjous.rpc.models import ContentType, Pingback


def get_post_from_target_uri(target_uri):
    """Takes a target URI and returns a Post, if one exists."""
    # We assume (for now) that the thing being pingback'd is a blog
    # post. This might be changed in future, but until then YAGNI swings
    # into play.
    target_pattern = (
        "/blog/(?P<date_posted__year>\d{4})/"
        "(?P<date_posted__month>\d{2})/"
        "(?P<date_posted__day>\d{2})/"
        "(?P<slug>[-\w]+)/$")
    target_path = urlsplit(target_uri)[2]
    target_match = re.match(target_pattern, target_path)

    # If the target URI matches, we look up the blog post.
    if target_match is not None:
        # Tidy up the match dict a bit - convert strings to ints. Hacky
        # hacky.
        match_dict = target_match.groupdict()
        date_fields = (
            'date_posted__day', 'date_posted__month', 'date_posted__year')
        for key in date_fields:
            match_dict[key] = int(match_dict[key])

        try:
            return Post.objects.get(**match_dict)
        except Post.DoesNotExist:
            raise NoSuchTargetURI("Target URI %s doesn't exist."
                % target_uri)
    else:
        # We should be a bit more fine grained here, but we don't care
        # too much about getting this wrong.
        raise InvalidTargetURI(
            "Target URI %s doesn't exist or can't receive pingbacks."
            % target_uri)


@xmlrpc_method(
    name='pingback.ping', returns='string', args=['string', 'string'])
def receive_pingback(source_uri, target_uri):
    """Receive an XML-RPC pingback from a remote server.

    See http://hixie.ch/specs/pingback/pingback#TOC3 for details of the
    pingback XML-RPC interface.
    """
    if not settings.PINGBACK_ENABLED:
        raise AccessDenied("Access denied.")

    target = get_post_from_target_uri(target_uri)

    # If the pingback has already been registered we raise a Fault.
    pingbacks = Pingback.objects.filter(
        source_uri=source_uri, object_id=target.id)
    if pingbacks.count() > 0:
        raise PingbackAlreadyRegistered(
            "Pingback from %s to %s is already registered." %
            (source_uri, target_uri))

    # Check that the source_uri exists...
    soup = BeautifulSoup(urlopen(source_uri))
    if soup.find(name='a', attrs={'href': target_uri}) is None:
        raise NonLinkingSourceURI(
            "Source URI %s doesn't link to %s." % (source_uri, target_uri))

    # Build the snippet. For now we just use the title of the remote
    # page or, failing that, the remote hostname.
    title = soup.find('title')
    hostname = urlsplit(source_uri)[1]
    if title is not None:
        snippet = "%s (%s)" % (title.renderContents().strip(), hostname)
    else:
        snippet = hostname

    # Finally, record the pingback.
    Pingback.objects.create(
        source_uri=source_uri, object_id=target.id, snippet=snippet,
        content_type=ContentType.objects.get_for_model(target.__class__))

    # If the request is successful we return a string response, per the
    # pingback spec.
    return "Pingback from %s to %s successful." % (source_uri, target_uri)


# We define some XML-RPC faults to deal with the faults defined in the
# pingback spec.

class PingbackFault(Fault):
    """Raised when no other fault code will do."""

    fault_code = 0

    def __init__(self, message=''):
        Fault.__init__(self, self.fault_code, message)


class NoSuchSourceURI(PingbackFault):
    """Raised when a given Source URI doesn't exist."""
    fault_code = 0x0010


class NonLinkingSourceURI(PingbackFault):
    """Raised when a Source URI doesn't link to a target URI."""
    fault_code = 0x0011


class NoSuchTargetURI(PingbackFault):
    """Raised when a target URI doesn't exist."""
    fault_code = 0x0020


class InvalidTargetURI(PingbackFault):
    """Raised when a target URI can't be pingback'd."""
    fault_code = 0x0021


class PingbackAlreadyRegistered(PingbackFault):
    """Raised when a pingback is already registered."""
    fault_code = 0x0030


class AccessDenied(PingbackFault):
    """Raised when access is denied."""
    fault_code = 0x0031


class CouldNotCommunicate(PingbackFault):
    """Raised when the client couldn't communicate with the remote server."""
    fault_code = 0x0032
