# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.

"""Views for the search app."""
import logging

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.cache import cache
from django.db.models import Q
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.defaultfilters import slugify

from frabjous.search.forms import SimpleSearchForm


# Load the list of models from settings
try:
    SEARCH_MODELS = [
        (ContentType.objects.get(name=cType).model_class(),
        fields, fieldnames)
        for cType, fields, fieldnames in settings.SEARCH_MODELS]
except AttributeError:
    SEARCH_MODELS = []


def perform_simple_search(terms):
    """Perform a search on a set of terms.

    :param terms: A list of term strings.
    """
    resdict = {}
    for model, fields, fieldnames in SEARCH_MODELS:
        # We build our request up as a series of Qs
        q = Q()
        for field in fields:
            field = '%s__contains' % field

            # We go through the list of terms to create our basic OR
            qList = [Q(**{field: term}) for term in terms]
            for query in qList:
                q = q | query

        # This should give us all model instances that contain terms
        # in fields.
        try:
            for res in model.objects.filter(q).distinct():
                rslt = {
                    'date': res.__getattribute__(
                        fieldnames['date_field']),
                    'body': res.__getattribute__(
                        fieldnames['body_field']),
                    'title': res.__getattribute__(
                        fieldnames['title_field']),
                    'url': res.get_absolute_url(),
                    'type': res._meta.object_name.lower()}

                # If we've got resdict for this date, make them a
                # list.
                if resdict.has_key(rslt['date']):
                    if type(resdict[rslt['date']]) is list:
                        resdict[date].append(rslt)
                    else:
                        resdict[rslt['date']] = [
                            resdict[rslt['date']], rslt]
                else:
                    resdict[rslt['date']] = rslt
        except Exception, e:
            log.error("Search failed %s" % str(e))

    return resdict


def simple_search(request):
    """Perform a search based on submitted criteria."""
    log = logging.getLogger('frabjous.search')

    # If it ain't a POST, we show them a search form or something.
    if not request.method == 'POST' or not request.POST.get('terms', False):
        # TODO - Change this
        return render_to_response('blog/index.html',
                                  context_instance=RequestContext(request))

    # we use a SimpleSearchForm to validate the incoming data
    form = SimpleSearchForm(request.POST)
    if not form.is_valid():
        log.error("Simple Search form is not valid: " + str(form.errors))
        return render_to_response(
            'blog/index.html', context_instance=RequestContext(request))

    # Assuming we get this far, we should have some terms with which to
    # work.
    clean_data = form.clean()
    terms = clean_data['terms']

    # We turn the terms into a list of something more useful. We search
    # on all terms in this simple search.
    terms_list = terms.split()

    # We can now get our results

    # Do we have a cached version of the results that we can return?
    results = cache.get('search.%s' % slugify(terms))
    if results is None:
        resdict = perform_simple_search(terms_list)

        # Now we sort the resdict by date (desc).
        resdates = resdict.keys()
        resdates.sort()
        resdates.reverse()

        # Finally we produce our final result set as a list of dicts
        results = []
        [results.append(resdict[i]) for i in resdates]

        # stick the results in the cache
        cache.set('search.%s' % slugify(terms), results, 60 * 15)

    log.debug('Returning %i results' % len(results))
    return render_to_response(
        'search/results.html', {'results': results, 'terms': terms},
        context_instance=RequestContext(request))

