# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.
"""Models for the tokens app."""

__metaclass__ = type
__all__ = [
    'RequesterCannotHaveToken',
    'Token',
    ]

import random

from datetime import datetime

from django.db import models
from django.db.models import permalink


# Possible token types.
TOKEN_TYPE_EMAIL = 1

TOKEN_TYPES = (
    (TOKEN_TYPE_EMAIL, "Email posting token"),
    )

URLIFYABLE_TOKEN_TYPES = []


def requester_can_have_token(requester, token_type):
    """Return true if a given requester can have a given token type."""
    # You can only have an email token if you're registered for email
    # posting (i.e. you're a valid user).
    if token_type == TOKEN_TYPE_EMAIL:
        from frabjous.blog.models import PostByEmailAlias
        try:
            user = PostByEmailAlias.objects.find_user_by_email(requester)
            return True
        except:
            return False


class RequesterCannotHaveToken(Exception):
    """Raised if a requester can't have a token."""


class TokenManager(models.Manager):
    """Manager for token creation."""

    def new(self, requester, token_type):
        """Generate a new token and return it.

        :param requester: The email address of the person requesting the
            token (or pseudo-email address if a real one doesn't exist.
        :param token_type: The type of token to create.
        :return: A new, unique token.

        A new token will be created of type `token_type` for the
        specified requester.
        >>> token = Token.objects.new(
        ...     'test@example.com', TOKEN_TYPE_EMAIL)
        >>> print token.requester
        test@example.com
        """
        characters = "0123456789bcdfghjklmnpqrstvwxyzBCDFGHJKLMNPQRSTVWXYZ"
        token_length = 20

        # Can the user have a token?
        if not requester_can_have_token(requester, token_type):
            raise RequesterCannotHaveToken()

        # If the token already exists, we create a new token.
        token_unique = False
        while not token_unique:
            token_string = ''.join(
                [random.choice(characters) for count in range(token_length)])

            try:
                token = self.get(pk=token_string)
            except:
                token = self.create(
                    pk=token_string, requester=requester, type=token_type)
                token_unique = True

        return token


class Token(models.Model):
    """A single-use token."""

    id = models.CharField(max_length=20, primary_key=True)
    type = models.IntegerField()
    date_created = models.DateTimeField(auto_now_add=True)
    date_consumed = models.DateTimeField(null=True, blank=True)
    requester = models.EmailField()

    # We use a TokenManager as our object manager.
    objects = TokenManager()

    @property
    def is_urlifyable(self):
        """Return True if the token can be turned into a URL."""
        return int(self.type) in URLIFYABLE_TOKEN_TYPES

    def consume(self):
        """Mark the token as consumed.

        consume()ing a token will set its date_consumed time.

        >>> token = Token.objects.new(
        ...     'test@example.com', TOKEN_TYPE_EMAIL)
        >>> print token.date_consumed
        None

        >>> token.consume()
        >>> token.date_consumed is None
        False
        """
        self.date_consumed = datetime.now()
        self.save()

    @permalink
    def get_absolute_url(self):
        """Return the absolute URL of the object."""
        return ('frabjous.tokens.views.consume_token', [str(self.id)])

    def __str__(self):
        return self.id
