# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.

"""Views for the tokens app."""
__metaclass__ = type
__all__ = [
    'generate_token',
    ]

from django.http import (
    HttpResponse, HttpResponseGone, HttpResponseForbidden,
    HttpResponseNotFound)
from django.shortcuts import render_to_response
from django.template import Context, Template
from django.template.loader import get_template

from frabjous.tokens.forms import GenerateTokenForm
from frabjous.tokens.models import (
    requester_can_have_token, RequesterCannotHaveToken, Token)


def generate_token(request):
    """View to generate a new token."""
    # If the request is a GET we just show the form rather than doing
    # any processing.
    if request.method == 'GET':
        return render_to_response(
            'tokens/generate.html', {'form': GenerateTokenForm()})

    form = GenerateTokenForm(request.POST)
    if not form.is_valid():
        return render_to_response(
            'tokens/generate.html', {'form': form})

    form.full_clean()
    post_data = form.cleaned_data

    can_have_token = (
        requester_can_have_token(
            post_data['requester'], int(post_data['token_type'])))

    if not can_have_token:
        # If the requester can't have a token, return a 403 (Forbidden).
        template = get_template('tokens/generate.html')
        context = Context(
            dict(message="That requester can't generate that token type.",
                 form=form))
        return HttpResponseForbidden(template.render(context))

    tokens =[
        Token.objects.new(
            requester=post_data['requester'],
            token_type=int(post_data['token_type']))
        for count in range(int(post_data['token_count']))]

    return render_to_response(
        'tokens/display.html',
        {'tokens': tokens, 'token_count': len(tokens)})


def consume_token(request, token_text):
    """Consume a token.

    If the token doesn't exist, return a 404. If it exists but has been
    used, return a 410 (Gone). Finally, if the token isn't one that can
    be consumed over HTTP, return a 403 (Forbidden).
    """
    template = get_template('tokens/consume.html')

    # First, try to get the token. Return a 404 if we can't find it.
    try:
        token = Token.objects.get(pk=token_text)
    except Token.DoesNotExist:
        context = Context(dict(
            error=True, messag="That token doesn't exist."))
        return HttpResponseNotFound(template.render(context))

    # If the token has already been consumed, raise a 410 (Gone).
    if token.date_consumed:
        context = Context(dict(
            error=True, message="This token has already been consumed."))
        return HttpResponseGone(template.render(context))

    # If the token can't be consumed over HTTP, return a 403.
    if not token.is_urlifyable:
        context = Context(dict(
            error=True, message="This token can't be consumed over HTTP."))
        return HttpResponseForbidden(template.render(context))

    # Consume the token and return a "Hoorah!" message.
    token.consume()
    context = Context(dict(message="Token consumed successfully."))
    return HttpResponse(template.render(context))
