# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.
"""A script to create posts from Twitter entries."""

import logging
import os
import sys

from datetime import datetime, timedelta

from django.conf import settings
from django.contrib.auth.models import User
from django.template.defaultfilters import slugify
from django.template.loader import render_to_string

from frabjous.blog.models import Post
from frabjous.categories.models import Category
from frabjous.remotefeeds.models import RemoteFeed


def run(args):
    # First, get all the posts for today, ordered by date, ascending
    today = datetime.now()
    last_week = datetime.now() - timedelta(days=7)

    post_title = "Dents for %s - %s" % (
        last_week.strftime("%Y-%m-%d"), today.strftime("%Y-%m-%d"))
    post_slug = slugify(post_title)

    # We only bother to do any of this if the post doesn't already
    # exist.
    existing_posts = Post.objects.filter(
        date_posted__year=today.year,
        date_posted__month=today.month,
        date_posted__day=today.day,
        slug=post_slug)
    if existing_posts.count() > 0:
        logging.info(
            "Not aggregating this week's posts; a post already exists.")
        return

    twitter_feed = RemoteFeed.objects.get(name='twitter')

    weeks_posts = twitter_feed.feedpost_set.filter(
        date__gte=last_week, date__lte=today)
    weeks_posts = weeks_posts.order_by('date')

    # If there aren't any posts for this week, quit.
    if weeks_posts.count() == 0:
        logging.debug(
            "Nothing to post for %s - %s." %
            (last_week.strftime("%Y-%m-%d"), today.strftime("%Y-%m-%d")))
        return

    for post in weeks_posts:
        logging.debug("%s: %s"  % (post.date, post.post_content))

    post_body = render_to_string(
        'scripts/twitter-post-body.html',
        {'date': today, 'tweets': weeks_posts})

    post = Post.objects.create(
        author=User.objects.get(pk=1),
        subject=post_title,
        slug=post_slug,
        body=post_body,
        published=False)

    # Categorise the post as 'Twitter'.
    twitter_cat = Category.objects.get(slug='twitter')
    post.categories.add(twitter_cat)
    post.save()
    twitter_cat.save()

    logging.info(
        "Posted aggregate post of %s entries: %s" % 
        (weeks_posts.count(), post.get_absolute_url()))


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.DEBUG,
        format='%(asctime)s %(levelname)s %(message)s',
        filename=os.path.join(
            settings.LOGGING_DIR, 'create-tweet-summaries.log')
        )

    run(sys.argv[:1])
