# Copyright (C) 2009    Graham Binns <graham@grahambinns.com>
#
# This file is part of the Frabjous blog and site engine.
#
# Frabjous is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Frabjous is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Frabjous. If not, see
# <http://www.gnu.org/licenses/>.

"""A script to create blog posts from email messages."""
import email
import logging
import os
import poplib
import re
import sys

from email.Utils import parseaddr

from django.contrib.auth.models import User
from django.conf import settings
from django.template.defaultfilters import slugify

from frabjous.blog.models import Post, PostByEmailAlias
from frabjous.categories.models import Category
from frabjous.tokens.models import Token, TOKEN_TYPE_EMAIL


def get_messages_to_handle():
    # Connect to the server and retrieve all the messages that haven't
    # already been retrieved.
    if settings.BLOG_POST_POP3_TLS:
        pop3_server = poplib.POP3_SSL(settings.BLOG_POST_POP3_SERVER)
    else:
        pop3_server = poplib.POP3(settings.BLOG_POST_POP3_SERVER)

    pop3_server.user(settings.BLOG_POST_POP3_USER)
    pop3_server.pass_(settings.BLOG_POST_POP3_PASS)

    response, message_list, octets = pop3_server.list()
    logging.debug("%i unread messages on the server." % len(message_list))
    messages = []
    for message in message_list:
        message_id, octets = message.split(' ')
        response, message_lines, octets = pop3_server.retr(message_id)

        message_string = "\n".join(message_lines)
        messages.append(email.message_from_string(message_string))
        pop3_server.dele(message_id)

    pop3_server.quit()
    return messages


def create_post_from_message(message, author):
    """Create a blog post from an `email.Message` instance."""

    tags = []
    post_lines = []

    # Loop over the lines in the message payload and strip out the ones
    # that we're not going to include in the post body.
    for message_line in message.get_payload().split('\n'):
        # Drop the token used to create the message.
        if message_line.lower().startswith('token:'):
            logging.debug("Ignoring token line")
            continue

        # If the line is a tags: field, we add the tags to the list of
        # tags (if we haven't done already).
        if message_line.lower().startswith('tags:'):
            logging.debug("Handling tags")
            tag_string = message_line.split(':')[1].strip()
            tags_in_string = [
                tag.lower().strip() for tag in tag_string.split(',')]

            tags = tags + [
                tag for tag in tags_in_string if tag not in tags]

            continue

        # If the line is a Signature indicator ('-- \n') we drop it and
        # anything after it.
        if message_line == '-- \n':
            break

        # Anything else can be included.
        post_lines.append(message_line)

    post_body = '\n'.join(post_lines).strip()

    logging.debug("Tags: %s" % str(tags))
    logging.debug("Post body:\n%s" % post_body)

    # So, now we create the Post.
    new_post = Post.objects.create(
        author=author, body=post_body, subject=message['subject'],
        slug=slugify(message['subject']))

    # Add the categories.
    for tag in tags:
        category, created = Category.objects.get_or_create(
            name=tag, slug=slugify(tag))

        new_post.categories.add(category)

    new_post.published = True
    new_post.save()

    logging.info("Created post %s." % new_post.get_absolute_url())


def process_message(message):
    """Process email message and turn them into blog posts.

    :param messages: A list of email.Message objects to turn into blog
        posts.
    """
    logging.warning("Just testing...")
    logging.debug("Processing message ID %s" % message['message-id'])

    # First things first, check that the sender is allowed to post by
    # email.
    logging.debug("Processing 'from' %s" % message['from']) 
    sender_name, email = parseaddr(message['from'])
    logging.debug("Sender is %s, email is %s" % (sender_name, email))
    try:
        logging.debug("Fetching user.")
        user = PostByEmailAlias.objects.find_user_by_email(email=email)
        logging.debug("Got user %s" % user.id)
    except User.DoesNotExist:
        logging.warning("Sender %s is not allowed to post by email." % email)
        return

    # Now, look for a token in the first line of the message payload.
    logging.debug("Looking for token")
    message_lines = message.get_payload().strip().split('\n')
    if not message_lines[0].lower().startswith('token:'):
        logging.warning("Message doesn't define a token.")
        return

    # Get the token, check it's valid.
    logging.debug("Checking token")
    token_text = message_lines[0].split(':')[1].strip()
    try:
        token = Token.objects.get(
            pk=token_text, requester=email, date_consumed__isnull=True,
            type=TOKEN_TYPE_EMAIL)
    except Token.DoesNotExist:
        logging.warning(
            "Message defines a token which doesn't exist or ism't valid.")
        return

    # Consume the token so it can't be used again.
    logging.debug("Consuming token %s." % token)
    token.consume()

    logging.debug("Creating post")
    create_post_from_message(message, user)


def run(args):
    messages_to_handle = get_messages_to_handle()
    for message in messages_to_handle:
        process_message(message)


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.DEBUG,
        format='%(asctime)s %(levelname)s %(message)s',
        filename=os.path.join(
            settings.LOGGING_DIR, 'process-email-posts.log')
        )

    run(sys.argv[:1])
